<?php
/*
 *  $Id$
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many individuals
 * and is licensed under the LGPL. For more information, see
 * <http://www.phpdoctrine.org>.
 */

/**
 * RangeVariableDeclaration ::= AbstractSchemaName ["AS"] AliasIdentificationVariable
 *
 * @author      Guilherme Blanco <guilhermeblanco@hotmail.com>
 * @license     http://www.opensource.org/licenses/lgpl-license.php LGPL
 * @link        http://www.phpdoctrine.org
 * @since       2.0
 * @version     $Revision$
 */
class Doctrine_ORM_Query_AST_RangeVariableDeclaration extends Doctrine_ORM_Query_AST
{
    private $_classMetadata;
    private $_abstractSchemaName;
    private $_aliasIdentificationVariable;

    public function __construct($classMetadata, $aliasIdentificationVar)
    {
        $this->_classMetadata = $classMetadata;
        $this->_abstractSchemaName = $classMetadata->getClassName();
        $this->_aliasIdentificationVariable = $aliasIdentificationVar;
    }    
    
    /* Getters */
    public function getAbstractSchemaName()
    {
        return $this->_abstractSchemaName;
    }

    public function getAliasIdentificationVariable()
    {
        return $this->_aliasIdentificationVariable;
    }

    public function getClassMetadata()
    {
        return $this->_classMetadata;
    }
    
    
    /* REMOVE ME LATER. COPIED METHODS FROM SPLIT OF PRODUCTION INTO "AST" AND "PARSER" */
    
    public function buildSql()
    {
        // Retrieving connection
        $conn = $this->_parserResult->getEntityManager()->getConnection();
        
        // Component alias
        $componentAlias = $this->_aliasIdentificationVariable;

        // Retrieving required information
        try {
            $queryComponent = $this->_parserResult->getQueryComponent($componentAlias);
            $classMetadata = $queryComponent['metadata'];
        } catch (Doctrine_Exception $e) {
            $this->_parser->semanticalError($e->getMessage());

            return;
        }

        return $conn->quoteIdentifier($classMetadata->getTableName()) . ' '
             . $conn->quoteIdentifier($this->_parserResult->getTableAliasFromComponentAlias($componentAlias));
    }
}